// Copyright (C) 2007 MadHyde
// Irrlicht Multilingual Project

#include "CIrrInputMethod.h"
#include "IrrlichtDevice.h"
#include <locale.h>
#ifdef WIN32
#include <windows.h>
#include <imm.h>
#endif

namespace irr
{

//! constructor
CIrrInputMethod::CIrrInputMethod(IrrlichtDevice* device)
: Device(device)
{
	if(Device)
		Device->grab();
}

//! destructor
CIrrInputMethod::~CIrrInputMethod()
{
	if(Device)
		Device->drop();
}

//! runs the device. Returns false if device wants to be deleted
bool CIrrInputMethod::run()
{
	MSG msg;
	bool quit = false;

	if(Device)
		Device->getTimer()->tick();

	while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);

		if(msg.message == WM_IME_COMPOSITION &&
			(msg.lParam & GCS_RESULTSTR) != 0 &&
			Device && Device->isWindowActive())
		{

			HWND hWND = GetFocus();
			HIMC hIMC = NULL;
			long mbstrLength = 0;
			unsigned char* mbstr = NULL;
			size_t wcstrLength = 0;
			wchar_t* wcstr = NULL;
			
			hIMC = ImmGetContext(hWND);
			mbstrLength = ImmGetCompositionString(hIMC, GCS_RESULTSTR, NULL, 0);
			mbstr = new unsigned char[mbstrLength + 1];
			ImmGetCompositionString(hIMC, GCS_RESULTSTR, mbstr, mbstrLength);
			mbstr[mbstrLength] = '\0';
			ImmReleaseContext(hWND, hIMC);

			char* old_locale = setlocale(LC_ALL, NULL);
			setlocale(LC_ALL,"");		// Use defalut locale

			wcstr = new wchar_t[mbstrLength + 1];
			wcstrLength = mbstowcs(wcstr, (char *)mbstr, mbstrLength);

			for(size_t i = 0; i < wcstrLength; i++)
			{
				irr::SEvent event;
				event.EventType = irr::EET_KEY_INPUT_EVENT;
				event.KeyInput.PressedDown = true;
				event.KeyInput.Char = wcstr[i]; //KeyAsc >= 0 ? KeyAsc : 0;
				event.KeyInput.Key = irr::KEY_ACCEPT;
				event.KeyInput.Shift = 0;
				event.KeyInput.Control = 0;
				Device->postEventFromUser(event);
			}

			setlocale(LC_ALL, old_locale);

			delete mbstr;
			delete wcstr;
		}
		else
		if (msg.message == WM_QUIT)
			quit = true;

	}

	_IRR_IMPLEMENT_MANAGED_MARSHALLING_BUGFIX;
	return !quit;
}

} // end namespace irr