/*
// Irrlicht Multilingual Project
// sample application
*/
#include <irrlicht.h>
#include <iostream>
#include "CGUITTFont.h"
#include "CGUIFileOpenDialogML.h"
#include "CGUIEditBoxML.h"
#include "CIrrInputMethod.h"
#ifdef WIN32
#include <windows.h>
#endif //WIN32


using namespace irr;

using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

#ifdef _IRR_WINDOWS_
#pragma comment(lib, "Irrlicht.lib")
#ifdef _DEBUG
#pragma comment(lib, "freetype235MT_D.lib")
#else
#pragma comment(lib, "freetype235MT.lib")
#endif
#pragma comment(lib, "imm32.lib")
#endif //_IRR_WINDOWS_


IrrlichtDevice *device = 0;
s32 cnt = 0;
IGUIListBox* listbox = 0;
CGUIFileOpenDialogML* fileopendialogML = 0;
IGUIEnvironment* env = 0;

class MyEventReceiver : public IEventReceiver
{
public:
	virtual bool OnEvent(const SEvent& event)
	{
		if (event.EventType == EET_GUI_EVENT)
		{
			s32 id = event.GUIEvent.Caller->getID();
			IGUIEnvironment* env = device->getGUIEnvironment();

			switch(event.GUIEvent.EventType)
			{

			case EGET_SCROLL_BAR_CHANGED:
				if (id == 104)
				{
					s32 pos = ((IGUIScrollBar*)event.GUIEvent.Caller)->getPos();
					
					for (u32 i=0; i<EGDC_COUNT ; ++i)
					{
						SColor col = env->getSkin()->getColor((EGUI_DEFAULT_COLOR)i);
						col.setAlpha(pos);
						env->getSkin()->setColor((EGUI_DEFAULT_COLOR)i, col);
					}
					
				}
				break;
			case EGET_BUTTON_CLICKED:

				if (id == 101)
				{
					device->closeDevice();
					return true;
				}

				if (id == 102)
				{
					listbox->addItem(L"Window created");
					cnt += 30;
					if (cnt > 200) 
						cnt = 0;

					IGUIWindow* window = env->addWindow(
						rect<s32>(100 + cnt, 100 + cnt, 300 + cnt, 200 + cnt), 
						false, // modal?
						L"Test window");

					env->addStaticText(L"Please close me",  
						rect<s32>(35,35,140,50),
						true, // border?
						false, // wordwrap?
						window);

					return true;
				}

				if (id == 103)
				{
					listbox->addItem(L"File open");
					fileopendialogML = new CGUIFileOpenDialogML(L"Please choose a file.", env, env->getRootGUIElement(), -1);
					fileopendialogML->drop();
//					env->addFileOpenDialog(L"Please choose a file.");
					return true;
				}

				break;
			default:
				break;
			}
		}

		return false;
	}
};


int main()
{
	// ask user for driver
	
	video::E_DRIVER_TYPE driverType;

	printf("Please select the driver you want for this example:\n"\
		" (a) Direct3D 9.0c\n (b) Direct3D 8.1\n (c) OpenGL 1.5\n"\
		" (d) Software Renderer\n (e) Burning's Software Renderer\n"\
		" (f) NullDevice\n (otherKey) exit\n\n");

	char i;
	std::cin >> i;

	switch(i)
	{
		case 'a': driverType = video::EDT_DIRECT3D9;break;
		case 'b': driverType = video::EDT_DIRECT3D8;break;
		case 'c': driverType = video::EDT_OPENGL;   break;
		case 'd': driverType = video::EDT_SOFTWARE; break;
		case 'e': driverType = video::EDT_BURNINGSVIDEO;break;
		case 'f': driverType = video::EDT_NULL;     break;
		default: return 1;
	}	

	// create device and exit if creation failed

	device = createDevice(driverType, core::dimension2d<s32>(640, 480));

	if (device == 0)
		return 1; // could not create selected driver.

	/* The creation was successful, now we set the event receiver and
		store pointers to the driver and to the gui environment. */

	MyEventReceiver receiver;
	device->setEventReceiver(&receiver);
	device->setWindowCaption(L"Irrlicht Engine - Multilingual GUI Demo");

	video::IVideoDriver* driver = device->getVideoDriver();
	env = device->getGUIEnvironment();

	CIrrInputMethod* irrIM = new CIrrInputMethod(device);

	/*
	To make the font a little bit nicer, we load an external font
	and set it as the new default font in the skin. 
	To keep the standard font for tool tip text, we set it to
	the built-in font.
	*/

	IGUISkin* skin = env->getSkin();

	CGUITTFace* face = new CGUITTFace();
	CGUITTFont* font = new CGUITTFont(env);
	c8 tmp[512];
	memset(tmp, 0, 512);
	sprintf(tmp, "%s", device->getFileSystem()->getWorkingDirectory());
	strcat(tmp,"\\fonts\\ipag.ttf");
//	GetWindowsDirectory(tmp,511);
//	strcat(tmp,"\\fonts\\msgothic.ttc");
	if(face->load(tmp))
	{
		font->attach(face, 12);
		skin->setFont(font);
	}
	else
		skin->setFont(env->getBuiltInFont());

	face->drop();
	font->drop();


	skin->setFont(env->getBuiltInFont(), EGDF_TOOLTIP);

	/*
	We add three buttons. The first one closes the engine. The second
	creates a window and the third opens a file open dialog. The third
	parameter is the id of the button, with which we can easily identify
	the button in the event receiver.
	*/	

	env->addButton(rect<s32>(10,210,110,210 + 32), 0, 101, L"Quit", L"Exits Program");
	env->addButton(rect<s32>(10,250,110,250 + 32), 0, 102, L"New Window", L"Launches a new Window");
	env->addButton(rect<s32>(10,290,110,290 + 32), 0, 103, L"File Open", L"Opens a file");

	/*
	Now, we add a static text and a scrollbar, which modifies the
	transparency of all gui elements. We set the maximum value of
	the scrollbar to 255, because that's the maximal value for 
	a color value.
	Then we create an other static text and a list box.
	*/

	env->addStaticText(L"Transparent Control:", rect<s32>(150,20,350,40), true);
	IGUIScrollBar* scrollbar = env->addScrollBar(true, rect<s32>(150, 45, 350, 60), 0, 104);
	scrollbar->setMax(255);

	// set scrollbar position to alpha value of an arbitrary element
	scrollbar->setPos(env->getSkin()->getColor(EGDC_WINDOW).getAlpha());

	env->addStaticText(L"Logging ListBox:", rect<s32>(50,80,250,100), true);
	listbox = env->addListBox(rect<s32>(50, 110, 250, 180));

	//env->addEditBox(L"Editable Text", rect<s32>(350, 80, 550, 100));
	CGUIEditBoxML* editboxML = new CGUIEditBoxML(L"Editable Text1", true,
		env, env->getRootGUIElement(), 0, rect<s32>(350, 80, 550, 100));
	editboxML->drop();

	editboxML = new CGUIEditBoxML(L"Editable Text2", true,
		env, env->getRootGUIElement(), 0, rect<s32>(350, 110, 550, 130));
	editboxML->drop();


	/*
	That's all, we only have to draw everything.
	*/

	while(irrIM->run() && driver)
	{
		// draw engine picture
		driver->beginScene(true, true, SColor(0,200,200,200));
		env->drawAll();
		driver->endScene();
	}

	irrIM->drop();
	device->drop();

	return 0;
}
